from typing import Optional
from pathlib import Path

from pydantic_settings import BaseSettings


class Settings(BaseSettings):
    """
    Settings class for application configuration.
    """

    LOG_FILE_PATH: Optional[str] = None
    LOG_LEVEL: str = "DEBUG"
    LOG_FORMAT: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    LOG_DATE_FORMAT: str = "%Y-%m-%d %H:%M:%S"
    LOG_MAX_BYTES: int = 10 * 1024 * 1024
    LOG_BACKUP_COUNT: int = 5

    PROVIDER: str = "openai"
    API_BASE: str = "https://api.openai.com/v1"
    API_KEY: str = ""
    MODEL: str = "gpt-3.5-turbo"

    PROMPT_TEMPLATE: str = (
        "请你根据用户的描述{{user_input}}，生成一个规范化的commit信息。请确保信息简洁明了，符合常规的commit规范。\n"
        + "修改的信息包括：{{diff_content}}。你的回复应该只包含规范化的commit信息，不需要其他内容。\n"
        + "请用英文回答，不要包含任何其他语言或注释。例如： feat: implement user authentication\n"
    )

    class Config:
        """
        Configuration for Pydantic settings.
        """

        env_file = str(Path.home() / ".config" / "cmai" / "settings.env")
        if not Path(env_file).exists():
            Path(env_file).parent.mkdir(parents=True, exist_ok=True)
            Path(env_file).touch()
        env_file_encoding = "utf-8"
        case_sensitive = False
        use_enum_values = True
        extra = "ignore"

    def load_from_env(self, env_file: Optional[str] = None):
        if env_file:
            self.Config.env_file = env_file
        self._load_env_file()

    def _load_env_file(self):
        env_file = Path(self.Config.env_file)
        if env_file.exists():
            with env_file.open(encoding=self.Config.env_file_encoding) as f:
                for line in f:
                    if line.strip() and not line.startswith("#"):
                        key, value = line.strip().split("=", 1)
                        setattr(self, key.strip(), value.strip())


settings = Settings()



import asyncio
from typing import Optional

import click

from cmai.config.settings import settings
from cmai.core.get_logger import LoggerFactory
from cmai.core.normalizer import Normalizer


async def normalize_commit_async(
    message: str, config: Optional[str] = None, repo: Optional[str] = None
):
    logger = LoggerFactory().get_logger("CMAI")

    if config:
        settings.load_from_env(config)

    logger.debug(f"Using configuration: {settings.model_dump_json(indent=2)}")
    logger.info(f"Normalizing commit message: {message}")

    normalizer = Normalizer()
    try:
        normalized_message = await normalizer.normalize_commit(
            user_input=message, prompt_template=settings.PROMPT_TEMPLATE, repo_path=repo
        )
        return normalized_message
    except Exception as e:
        logger.error(f"Error normalizing commit message: {e}")
        raise click.ClickException(f"Failed to normalize commit message: {e}")


@click.command()
@click.argument("message", type=str, required=True)
@click.option("--config", "-c", help="配置文件路径", default=None, type=str)
@click.option("--repo", "-r", help="Git仓库路径", default=None, type=str)
def main(message: str, config: str, repo: str):
    """将口语化的commit信息规范化"""
    try:
        result = asyncio.run(normalize_commit_async(message, config, repo))
        content = result.content
        token_usage = result.tokens_used

        click.echo(click.style(f"Commit message: {content}", fg="green"))
        click.echo(click.style(f"Tokens used: {token_usage}", fg="blue"))
    except Exception as e:
        raise click.ClickException(str(e))


if __name__ == "__main__":
    main()



from typing import Optional

from cmai.utils.git_staged_analyzer import GitStagedAnalyzer
from cmai.providers.base import BaseAIClient, AIResponse
from cmai.providers.bailian_provider import BailianProvider


class Normalizer:
    def __init__(self) -> None:
        pass

    async def normalize_commit(
        self,
        user_input: str,
        prompt_template: str,
        repo_path: Optional[str] = None,
        provider: BaseAIClient = BailianProvider(),
    ) -> AIResponse:
        git_analyzer = GitStagedAnalyzer(repo_path=repo_path)
        cached_diff = git_analyzer.get_cached_diff()

        if not cached_diff or len(cached_diff) == 0:
            raise ValueError("No staged changes found in the repository.")

        diff_content = "\n".join(cached_diff)
        prompt = prompt_template.replace("{user_input}", user_input).replace(
            "{diff_content}", diff_content
        )

        response = await provider.normalize_commit(prompt)

        return response



import subprocess
from typing import List, Optional
from pathlib import Path

from cmai.core.get_logger import LoggerFactory


class GitStagedAnalyzer:
    def __init__(self, repo_path: Optional[str] = None) -> None:
        self.logger = LoggerFactory().get_logger("GitAddLogger")
        if repo_path is None:
            repo_path = str(Path.cwd().resolve())
        self.logger.info(f"Initializing GitStagedAnalyzer for repo: {repo_path}")
        self.repo_path = Path(repo_path).resolve()

    def get_cached_diff(self) -> List[str]:
        try:
            stat_result = subprocess.run(
                ["git", "diff", "--cached", "--numstat"],
                cwd=self.repo_path,
                capture_output=True,
                text=True,
                check=True,
            )
            return stat_result.stdout.strip().splitlines()
        except subprocess.CalledProcessError as e:
            self.logger.error(f"Error checking git status: {e}")
            return []



from abc import ABC, abstractmethod
from typing import Optional

from pydantic import BaseModel


class AIResponse(BaseModel):
    """
    Base class for AI response models.
    """

    content: str
    model: str
    provider: str
    tokens_used: Optional[int] = None


class BaseAIClient(ABC):
    """AI客户端抽象基类"""

    def __init__(
        self, api_key: Optional[str] = None, model: Optional[str] = None, **kwargs
    ) -> None:
        """
        初始化AI客户端

        Args:
            api_key (Optional[str], optional): API密钥，不传入则尝试从环境变量获取.
            Defaults to None.
            model (Optional[str], optional): 使用的模型名称，不传入则使用默认模型.
            Defaults to None.
            **kwargs: 其他可选参数，具体取决于子类实现.
        """
        self.api_key = api_key
        self.model = model
        self.kwargs = kwargs

    @abstractmethod
    async def normalize_commit(self, prompt: str, **kargs) -> AIResponse:
        """
        规范化commit消息的抽象方法

        Args:
            prompt (str): 用户输入的提示信息，通常包含待规范化的commit消息和相关上下文信息.

        Returns:
            AIResponse: 包含规范化commit消息的响应对象.
        """
        raise NotImplementedError(
            "Subclasses must implement the normalize_commit method."
        )

    @abstractmethod
    def validate_config(self) -> bool:
        """
        验证配置是否正确，**不会检测API密钥的有效性。**

        如果API密钥无效，会在实际使用中抛出异常。

        Returns:
            bool: 如果配置正确返回True，否则返回False.
        """
        raise NotImplementedError(
            "Subclasses must implement the validate_config method."
        )



import logging
import sys
from logging import Logger
from pathlib import Path

from cmai.config.settings import settings


class LoggerFactory:
    _instance: "LoggerFactory"
    _loggers: dict[str, Logger]
    _stream_loggers: dict[str, Logger]
    _log_file: Path

    def __new__(cls, *args, **kwargs) -> "LoggerFactory":
        if not hasattr(cls, "_instance"):
            cls._instance = super(LoggerFactory, cls).__new__(cls, *args, **kwargs)
            cls._loggers = {}
            cls._stream_loggers = {}
            if not settings.LOG_FILE_PATH:
                log_file_path = Path.home() / ".logs" / "cmai" / "cmai.log"
            elif Path.exists(Path(settings.LOG_FILE_PATH)):
                log_file_path = Path(settings.LOG_FILE_PATH)
            else:
                log_file_path = Path.home() / ".logs" / "cmai" / "cmai.log"
            cls._log_file = Path(log_file_path).resolve()
        return cls._instance

    def get_logger(self, name: str) -> Logger:
        if not self._log_file.exists():
            self._log_file.parent.mkdir(parents=True, exist_ok=True)
            self._log_file.touch()

        if name not in self._loggers:
            logger = logging.getLogger(name)
            logger.setLevel(settings.LOG_LEVEL)

            formatter = logging.Formatter(
                settings.LOG_FORMAT, datefmt=settings.LOG_DATE_FORMAT
            )

            file_handler = logging.FileHandler(LoggerFactory._log_file, mode="a")
            file_handler.setLevel(settings.LOG_LEVEL)
            file_handler.setFormatter(formatter)

            stream_handler = logging.StreamHandler()
            stream_handler.setLevel(settings.LOG_LEVEL)
            stream_handler.setFormatter(formatter)

            logger.addHandler(file_handler)
            logger.addHandler(stream_handler)

            self._loggers[name] = logger

        return self._loggers[name]

    def get_stream_logger(self, name: str, level=logging.INFO) -> Logger:
        """获取自定义流处理器的日志记录器"""
        if name in self._stream_loggers:
            return self._stream_loggers[name]

        logger = self.get_logger(name)
        logger.setLevel(level)
        logger.handlers.clear()

        stream_handler = StreamHandler(sys.stdout)
        stream_handler.setFormatter(logging.Formatter("%(message)s"))
        logger.addHandler(stream_handler)
        logger.propagate = False

        self._stream_loggers[name] = logger

        return logger


class StreamHandler(logging.StreamHandler):
    """自定义流处理器，用于实时输出"""

    def emit(self, record):
        msg = self.format(record)
        stream = self.stream
        stream.write(msg)
        stream.flush()



from typing import Optional
import os

from openai import OpenAI

from cmai.config.settings import settings
from cmai.providers.base import BaseAIClient, AIResponse
from cmai.core.get_logger import LoggerFactory


class BailianProvider(BaseAIClient):
    """百炼AI客户端实现"""

    def __init__(
        self,
        api_key: Optional[str] = None,
        model: Optional[str] = None,
        **kwargs,
    ) -> None:
        """
        初始化百炼AI客户端

        Args:
            api_key (str): API密钥，用于认证请求，默认从环境变量中获取。
            model (str, optional): 使用的模型名称，默认为"qwen-turbo-latest"。
            **kwargs: 其他可选参数，具体取决于百炼AI的API配置。
        """
        super().__init__(api_key=api_key, model=model, **kwargs)
        self.logger = LoggerFactory().get_logger("BailianProvider")
        self.stream_logger = LoggerFactory().get_stream_logger("BailianProviderStream")

        # 尝试从环境变量或配置文件中获取API Key
        if not self.api_key:
            env_api_key = os.getenv("DASHSCOPE_API_KEY")
            self.api_key = env_api_key if env_api_key else settings.API_KEY

        # 验证API Key是否存在
        if not self.api_key:
            raise ValueError("API key is required for BailianProvider.")

        # 获取 API Base
        if not settings.API_BASE:
            base_url = "https://dashscope.aliyuncs.com/compatible-mode/v1"
        else:
            base_url = settings.API_BASE

        # 获取 model
        if not model:
            model = settings.MODEL or "qwen-turbo-latest"

        self.model = model

        self.client = OpenAI(api_key=self.api_key, base_url=base_url, **kwargs)

    def validate_config(self) -> bool:
        return bool(self.api_key)

    async def normalize_commit(self, prompt: str, **kargs) -> AIResponse:
        self.logger.debug(f"Normalizing commit with prompt: {prompt}")

        completion = self.client.chat.completions.create(
            model=self.model or "qwen-turbo-latest",
            messages=[{"role": "user", "content": prompt}],
            stream=True,
            stream_options={"include_usage": True},
            **kargs,
        )

        reason = ""
        response = ""
        is_answering = False
        is_reasoning = False
        usage = None
        for chunk in completion:
            if chunk.choices:
                delta = chunk.choices[0].delta
                if (
                    hasattr(delta, "reasoning_content")
                    and getattr(delta, "reasoning_content", None) is not None
                ):
                    if not is_reasoning:
                        self.logger.info(
                            "Detected reasoning content...\nPlease wait..."
                        )
                        is_reasoning = True
                    self.stream_logger.info(getattr(delta, "reasoning_content", ""))
                    reason += getattr(delta, "reasoning_content", "")
                else:
                    if not is_answering:
                        self.stream_logger.info("\n")
                        self.logger.debug("Starting to answer...")
                        is_answering = True
                    self.stream_logger.info(chunk.choices[0].delta.content or "")
                    response += chunk.choices[0].delta.content or ""
            elif chunk.usage:
                usage = chunk.usage.total_tokens
                self.logger.debug(f"Received usage info: {usage} tokens")
            else:
                self.logger.warning(f"Unexpected chunk received: {chunk}")

        if usage is None:
            self.logger.warning("No usage information received")
            usage = 0

        self.logger.info(f"Final normalized commit message: {response.strip()}")

        return AIResponse(
            content=response.strip(),
            model=self.model or "qwen-turbo-latest",
            provider="bailian",
            tokens_used=usage,
        )
